import createDebugger from "debug";
import { ModuleTree } from "./module-tree.js";
const debug = createDebugger("counterfact:server:registry");
function castParameters(parameters, parameterTypes) {
    const copy = { ...parameters };
    Object.entries(copy).forEach(([key, value]) => {
        copy[key] =
            parameterTypes?.[key] === "number"
                ? // eslint-disable-next-line @typescript-eslint/consistent-type-assertions
                    Number.parseInt(value, 10)
                : value;
    });
    return copy;
}
export class Registry {
    moduleTree = new ModuleTree();
    get routes() {
        return this.moduleTree.routes;
    }
    add(url, module) {
        this.moduleTree.add(url, module);
    }
    remove(url) {
        this.moduleTree.remove(url);
    }
    exists(method, url) {
        return Boolean(this.handler(url).module?.[method]);
    }
    handler(url) {
        const match = this.moduleTree.match(url);
        return {
            matchedPath: match?.matchedPath ?? "",
            module: match?.module,
            path: match?.pathVariables ?? {},
        };
    }
    endpoint(httpRequestMethod, url, parameterTypes = {}) {
        const handler = this.handler(url);
        debug("handler for %s: %o", url, handler);
        const execute = handler.module?.[httpRequestMethod];
        if (!execute) {
            return () => ({
                body: `Could not find a ${httpRequestMethod} method matching ${url}\n`,
                contentType: "text/plain",
                headers: {},
                status: 404,
            });
        }
        return async ({ ...requestData }) => {
            const operationArgument = {
                ...requestData,
                headers: castParameters(requestData.headers, parameterTypes.header),
                matchedPath: handler.matchedPath,
                path: castParameters(handler.path, parameterTypes.path),
                query: castParameters(requestData.query, parameterTypes.query),
            };
            // eslint-disable-next-line id-length
            operationArgument.x = operationArgument;
            return await execute(operationArgument);
        };
    }
}
